local Commands = {
    updateVehicleAlarmState = function(args)
        local vehicle = getVehicleById(args.vehicle)
        if not vehicle then return end
        vehicle:setAlarmed(args.alarm)
    end,
    updateVehicleAlarmInstalled = function(args)
        local vehicle = getVehicleById(args.vehicle)
        if not vehicle then return end
        vehicle:getModData().hasAlarm = args.hasAlarm
    end,
    updateVehicleEngine = function(args)
        local vehicle = getVehicleById(args.vehicle)
        if not vehicle then return end
        vehicle:setEngineFeature(args.quality, args.noise, args.power)
        vehicle:setMaxSpeed(args.maxSpeed)
        vehicle:getModData().hasAlarm = args.hasAlarm
    end,
    updateVehicleRust = function(args)
        local vehicle = getVehicleById(args.vehicle)
        if not vehicle then return end
        vehicle:setRust(args.rust)
    end,
}

Events.OnServerCommand.Add(function(module, command, args)
    if not isClient() then return end
	if module == 'pimpmyride' and Commands[command] then
		Commands[command](args or {})
	end
end)


local function pimpThisRide(worldobjects, vehicle, playerObj)
    if isClient() then
        sendClientCommand(playerObj, "pimpmyride", "pimpThisRide", { vehicle = vehicle:getId() })
    else
        vehicle:repair()
        local engineLoudness = vehicle:getScript():getEngineLoudness() or 100
        engineLoudness = engineLoudness * (SandboxVars.ZombieAttractionMultiplier or 1)
        vehicle:setEngineFeature(100, engineLoudness, 6000)
        vehicle:setMaxSpeed(120)
        vehicle:getModData().hasAlarm = true
    end
end
local function setAlarmState(worldobjects, vehicle, playerObj, value)
    if isClient() then
        sendClientCommand(playerObj, "pimpmyride", "setAlarmState", { vehicle = vehicle:getId(), alarm = value })
    else
        vehicle:setAlarmed(value)
    end
end
local function setAlarmInstalled(worldobjects, vehicle, playerObj, value)
    if isClient() then
        sendClientCommand(playerObj, "pimpmyride", "setAlarmInstalled", { vehicle = vehicle:getId(), hasAlarm = value })
    else
        vehicle:getModData().hasAlarm = value
    end
end

local function setRust(worldobjects, vehicle, playerObj, value)
    if isClient() then
        sendClientCommand(playerObj, "pimpmyride", "setRust", { vehicle = vehicle:getId(), rust = value })
    else
        vehicle:setRust(value)
    end
end
local function reloadVehicle(worldobjects, vehicle, playerObj)
    if isClient() then
        sendClientCommand(playerObj, "pimpmyride", "reloadScript", { vehicle = vehicle:getId() })
    else
        vehicle:scriptReloaded()
    end
end


-------------------------------------------------------------------------------------------
-- context menu handling

local oldFillMenuOutsideVehicle =ISVehicleMenu.FillMenuOutsideVehicle
function ISVehicleMenu.FillMenuOutsideVehicle(player, context, vehicle, test)
	local playerObj = getSpecificPlayer(player)
    oldFillMenuOutsideVehicle(player, context, vehicle, test)
    local modData = vehicle:getModData()
    if isAdmin() or not isClient() then
        local pimpMenu = context:addOption("Pimp My Ride!", playerObj, nil)
        local subMenuPimp = context:getNew(context)
        context:addSubMenu(pimpMenu, subMenuPimp)
        subMenuPimp:addOption("Pimp The Engine", playerObj, pimpThisRide, vehicle, playerObj)
        subMenuPimp:addOption("Remove Rust", playerObj, setRust, vehicle, playerObj, 0.0)
        subMenuPimp:addOption("Add Rust", playerObj, setRust, vehicle, playerObj, 0.9)
        if modData.hasAlarm then
            subMenuPimp:addOption("Uninstall Alarm", playerObj, setAlarmInstalled, vehicle, playerObj, false)
        else
            subMenuPimp:addOption("Install Alarm", playerObj, setAlarmInstalled, vehicle, playerObj, true)
        end
        --subMenuPimp:addOption("Reload Vehicle", playerObj, reloadVehicle, vehicle, playerObj)
        context:addOption("Roadtrip UI", playerObj, ISVehicleMenu.onRoadtrip)
        
    end
    if playerObj:getInventory():haveThisKeyId(vehicle:getKeyId()) and modData.hasAlarm then
        if vehicle:isAlarmed() then
            context:addOption("Disable Alarm", playerObj, setAlarmState, vehicle, playerObj, false)
        else
            context:addOption("Enable Alarm", playerObj, setAlarmState, vehicle, playerObj, true)
        end
    end
end
